﻿using MediatR;
using Microsoft.Extensions.Logging;
using NetworkResurrector.Agent.Application.Services.Abstractions;
using NetworkResurrector.Agent.PublishedLanguage.Commands;
using NetworkResurrector.Agent.PublishedLanguage.Events;
using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;

namespace NetworkResurrector.Agent.Application.CommandHandlers
{
    public class ShutdownHandler : IRequestHandler<Shutdown, ShutdownResult>
    {
        private readonly ILogger<ShutdownHandler> _logger;
        private readonly IShutdownService _shutdownService;

        public ShutdownHandler(ILogger<ShutdownHandler> logger, IShutdownService shutdownService)
        {
            _logger = logger;
            _shutdownService = shutdownService;
        }

        public async Task<ShutdownResult> Handle(Shutdown command, CancellationToken cancellationToken)
        {
            return await Task.Run(() => Handle(command));
        }

        private ShutdownResult Handle(Shutdown command)
        {
            _logger.LogDebug($"Start shutting down the system.");

            var stopWatch = new Stopwatch();
            stopWatch.Start();

            _shutdownService.Run();

            stopWatch.Stop();
            _logger.LogDebug($"System shutdown finished - {stopWatch.ElapsedMilliseconds:N0} ms");

            return new ShutdownResult() { Success = true };
        }
    }
}
